const St = imports.gi.St;
const Main = imports.ui.main;
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;
const AppDisplay = imports.ui.appDisplay;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Lib = Extension.imports.lib;
const Windows = Extension.imports.windows;

const schema = "org.gnome.shell.extensions.TaskBar";

function init(extensionMeta)
{
    return new TaskBar(extensionMeta, schema);
}

function TaskBar(extensionMeta, schema)
{
    this.init(extensionMeta, schema);
}

TaskBar.prototype =
{
    extensionMeta: null,
    saveApplicationMenu: null,
    boxMain: null,
    tasksList: new Array(),
    desktopView: null,

    init: function(extensionMeta, schema)
    {
	this.extensionMeta = extensionMeta;
	this.schema = schema;
    },

    onClickDesktopButton: function(button, pspec)
    {
	this.tasksList.forEach(
	    function(task)
	    {
		if (this.desktopView)
		    task[0].unminimize(global.get_current_time());
		else
		    task[0].minimize(global.get_current_time());
	    },
	    this
	);

	this.desktopView = ! this.desktopView;
    },

    onClickTaskButton: function(button, pspec, window)
    {
	this.tasksList.forEach(
	    function(task)
	    {
		if (task[0] == window)
		{
		    if (! task[0].has_focus())
		    {
			task[0].activate(global.get_current_time());
			task[1].add_style_pseudo_class('active');
		    }
		    else
		    {
			task[0].minimize(global.get_current_time());
			task[1].remove_style_pseudo_class('active');
		    }
		}
		else
		    task[1].remove_style_pseudo_class('active');
	    },
	    this
	);

	this.desktopView = false;
    },
   
    onWindowsListChanged: function(windowsList, type, window)
    {
	this.desktopView = false;

	if (type == 0) //Add all windows (On init or workspace change)
	{
	    for (let i = this.tasksList.length - 1; i>=0; i--)
	    {
		let task = this.tasksList[i];

		task[1].disconnect(task[2]);
		task[1].destroy();
		this.tasksList.splice(i, 1);
	    };

	    windowsList.forEach(
		function(win)
		{
		    let app = Shell.WindowTracker.get_default().get_window_app(win);
		    let iconTask = new AppDisplay.AppIcon(app, { setSizeManually: true, showLabel: false });
		    iconTask.setIconSize(24);

		    let buttonTask = new St.Button({ style_class: "tkb-task-button" });
		    let signalTask = buttonTask.connect("clicked", Lang.bind(this, this.onClickTaskButton, win));

		    if (win.has_focus())
			buttonTask.add_style_pseudo_class('active');

		    buttonTask.set_child(iconTask.actor);

		    this.boxMain.add_actor(buttonTask);
		    
		    this.tasksList.push([ win, buttonTask, signalTask ]);
		},
		this
	    );

	    this.desktopView = false;
	}
	else if (type == 1) //Add window
	{
	    let app = Shell.WindowTracker.get_default().get_window_app(window);
	    let iconTask = new AppDisplay.AppIcon(app, { setSizeManually: true, showLabel: false });
	    iconTask.setIconSize(24);

	    let buttonTask = new St.Button({ style_class: "tkb-task-button" });
	    let signalTask = buttonTask.connect("clicked", Lang.bind(this, this.onClickTaskButton, window));
	    buttonTask.set_child(iconTask.actor);

	    this.boxMain.add_actor(buttonTask);

	    this.tasksList.push([ window, buttonTask, signalTask ]);

	    this.desktopView = false;
	}
	else if (type == 2) //Remove window
	{
	    for (let indexTask in this.tasksList)
	    {
		let task = this.tasksList[indexTask];

		if (task[0] == window)
		{
		    task[1].disconnect(task[2]);
		    task[1].destroy();
		    this.tasksList.splice(indexTask, 1);

		    break;
		}
	    };
	}
    },

    onWindowChanged: function(window, type)
    {
	if (type == 0) //Focus
	{
	    this.tasksList.forEach(
		function(task)
		{
		    if (task[0] == window)
			task[1].add_style_pseudo_class('active');
		    else
			task[1].remove_style_pseudo_class('active');
		},
		this
	    );
	}

	this.desktopView = false;
    },

    enable: function()
    {
	this.saveApplicationMenu = Main.panel._appMenu.actor;
	Main.panel._leftBox.remove_actor(this.saveApplicationMenu);

	this.boxMain = new St.BoxLayout();
	Main.panel._leftBox.insert_child_at_index(this.boxMain, Main.panel._leftBox.get_children().length);

        let iconDesktop = new St.Icon(
            {
		gicon: Gio.icon_new_for_string(this.extensionMeta.path + "/images/desktop.png"),
                icon_type: St.IconType.SYMBOLIC,
		icon_size: 24,
		style_class: "tkb-desktop-icon"
            });
	let buttonDesktop = new St.Button({ style_class: "tkb-task-button" });
	let signalDesktop = buttonDesktop.connect("clicked", Lang.bind(this, this.onClickDesktopButton));
	buttonDesktop.set_child(iconDesktop);

	let boxDesktop = new St.BoxLayout({ style_class: "tkb-desktop-box" });
	boxDesktop.add_actor(buttonDesktop);

	this.boxMain.add_actor(boxDesktop);

	this.windows = new Windows.Windows(this, this.onWindowsListChanged, this.onWindowChanged);
    },

    disable: function()
    {
	Main.panel._leftBox.remove_actor(this.boxMain);
	this.boxMain = null;
	this.tasksList = new Array();

	Main.panel._leftBox.add(this.saveApplicationMenu);
    }
}
